package com.yizhi.application.task;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yizhi.application.controller.UserInfoController;
import com.yizhi.application.domain.PublicPlatformConfig;
import com.yizhi.application.domain.TrAccountUserinfo;
import com.yizhi.application.domain.UserInfo;
import com.yizhi.application.feign.WeChatClient;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.application.service.ITrAccountUserinfoService;
import com.yizhi.application.service.IUserInfoService;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.system.application.system.remote.OrganizationClient;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.system.application.vo.WeChatUpdateOrganizationVo;
import com.yizhi.wechat.application.vo.wechat.Constants;
import com.yizhi.wechat.application.vo.wechat.WeChatUserVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * 更新用户信息
 * 2020-2-10
 *
 * @author lingye
 */
@Component
public class UpdateUserInfoService {

    private static final Logger LOGGER = LoggerFactory.getLogger(UpdateUserInfoService.class);
    @Autowired
    private AccountClient accountClient;
    @Autowired
    private TaskExecutor taskExecutor;
    @Autowired
    private UserInfoController userInfoController;
    @Autowired
    IdGenerator idGenerator;
    @Autowired
    private WeChatClient weChatClient;
    @Autowired
    private IUserInfoService userInfoService;
    @Autowired
    private ITrAccountUserinfoService trAccountUserinfoService;
    @Autowired
    private OrganizationClient remoteOrganizationClient;

    /**
     * 异步更新用户信息
     *
     * @param publicPlatformConfig
     * @param accountUserInfo
     */
    public void taskUpdateUserInfo(PublicPlatformConfig publicPlatformConfig, TrAccountUserinfo accountUserInfo) {
        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                updateUserInfo(publicPlatformConfig, accountUserInfo);
            }
        });
    }

    private void updateUserInfo(PublicPlatformConfig publicPlatformConfig, TrAccountUserinfo accountUserInfo) {
        LOGGER.info("异步更新用户信息===============start=============");
        String accessToken = null;
        try {
            trAccountUserinfoService.autoUnBindWeChatAccount(accountUserInfo);

            if (StringUtils.isNotEmpty(publicPlatformConfig.getAgentId())) {
                LOGGER.info("异步更新用户信息公众号配置：" + publicPlatformConfig);
                accessToken = userInfoController.getWeChatAccessToken(publicPlatformConfig);
                LOGGER.info(" 获取企业的accessToken：" + accessToken);
                // 查询用户的信息
                WeChatUserVO weChatUserVO = userInfoController.getWechatUser(accessToken, accountUserInfo.getWechatuserid());
                //更新用户信息
                updateAccountInfo(accountUserInfo.getWechatuserid(), weChatUserVO.getAccountVO().getFullName(), weChatUserVO.getAccountVO().getHeadPortrait());
                //更新部门
                updateOrg(accountUserInfo, weChatUserVO);

            }
        } catch (Exception e) {
            e.printStackTrace();
            LOGGER.info("企业信息更新失败！{}", e.getMessage());
        }
        LOGGER.info("异步更新用户信息===============end=============");
    }

    /**
     * //更新部门
     */
    private void updateOrg(TrAccountUserinfo accountUserInfo, WeChatUserVO weChatUserVO) {
        if (accountUserInfo == null || weChatUserVO == null) {
            LOGGER.error("更新部门的必要vo参数为空:{}");
            return;
        }
        Long companyId = accountUserInfo.getCompanyId();
        Long accountId = accountUserInfo.getAccountId();
        List<String> orgNames = weChatUserVO.getOrgNames();
        if (companyId == null || accountId == null || CollectionUtils.isEmpty(orgNames)) {
            LOGGER.error("更新部门的必要参数为空:{}" + companyId + "--" + accountId + "--" + orgNames);
            return;
        } else {
            WeChatUpdateOrganizationVo vo = new WeChatUpdateOrganizationVo();
            vo.setAccountId(accountId);
            vo.setCompanyId(companyId);
            vo.setOrgNames(orgNames);
            LOGGER.info("更新部门的传参:{}" + JSON.toJSONString(vo));
            String reason = remoteOrganizationClient.updateOrganization(vo).getReason();
            LOGGER.info("更新部门的结果:{}" + reason);
        }
    }

    /**
     * 异步新增用户信息
     *
     * @param accessToken
     * @param weChatUserId
     */
    public void taskAddUserInfo(String accessToken, String weChatUserId, String unionId) {
        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                addUserInfo(accessToken, weChatUserId, unionId);
            }
        });
    }

    private void addUserInfo(String accessToken, String weChatUserId, String unionId) {
        try {
            JSONObject userInfoJSON = weChatClient.getUserInfo(accessToken, weChatUserId);
            LOGGER.info("用户详细信息{}", userInfoJSON);
            int userCount = getUserCount(weChatUserId);
            if (userCount == 0) {
                addUser(userInfoJSON);
            } else {

                TrAccountUserinfo accountUserInfo = new TrAccountUserinfo();
                accountUserInfo.setWechatuserid(weChatUserId);
                accountUserInfo.setIsDeleted(Constants.DEL_FLAG);
                List<TrAccountUserinfo> accountUserinfoList = trAccountUserinfoService.selectList(QueryUtil.condition(accountUserInfo));
                if (CollectionUtils.isNotEmpty(accountUserinfoList)) {
                    TrAccountUserinfo trAccountUserinfo = accountUserinfoList.get(0);

                    AccountVO accountVO = new AccountVO();
                    accountVO.setId(trAccountUserinfo.getAccountId());
                    String headImage = userInfoJSON.getString("headimgurl");
                    String nickName = userInfoJSON.getString("nickname");
                    accountVO.setHeadPortrait(headImage);
                    Boolean updateAccountResult = accountClient.updateAccount(accountVO);
                    LOGGER.info("异步更新用用户头像：{}", (updateAccountResult ? "成功" : "失败"));
                    updateAccountInfo(weChatUserId, nickName, headImage);
                }


            }
        } catch (Exception e) {
            LOGGER.info("获取信用户的详情异常:{}", e.getLocalizedMessage());
        }
    }

    public int getUserCount(String openId) {
        UserInfo userInfo = new UserInfo();
        userInfo.setWechatuserid(openId);
        userInfo.setIsDeleted(Constants.DEL_FLAG);
        int count = userInfoService.selectCount(QueryUtil.condition(userInfo));
        return count;
    }

    @Transactional(rollbackFor = Exception.class)
    public void addUser(JSONObject user) {
        UserInfo userInfo = new UserInfo();
        userInfo.setId(idGenerator.generate());
        userInfo.setWechatuserid(user.getString("openid") != null ? user.getString("openid") : null);
        userInfo.setNickname(user.getString("nickname") != null ? user.getString("nickname") : null);
        userInfo.setProvince(user.getString("province") != null ? user.getString("province") : null);
        userInfo.setCity(user.getString("city") != null ? user.getString("city") : null);
        userInfo.setHeadimgurl(user.getString("headimgurl") != null ? user.getString("headimgurl") : null);
        userInfo.setPrivilege(user.getString("privilege") != null ? user.getString("privilege") : null);
        userInfo.setUnionId(user.getString("unionid") != null ? user.getString("unionid") : null);
        userInfo.setCreateTime(new Date());
        userInfo.setUserType(Constants.YES_FLAG);

        boolean insertResult = userInfoService.insert(userInfo);
        LOGGER.info("微信用户新增结果：{}", insertResult ? "成功" : "失败");
    }

    /**
     * 新增企业用户
     *
     * @param userDetailJON
     */
    public void taskAddCompanyUserInfo(JSONObject userDetailJON) {
        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                saveCompanyUserInfo(userDetailJON);
            }
        });
    }

    /**
     * 企业用户新增
     *
     * @param userDetailJON
     */
    private void saveCompanyUserInfo(JSONObject userDetailJON) {
        String userId = userDetailJON.getString("userid");
        //异步修改用户信息
        try {
            int userCount = getCompanyUserCount(userId);
            if (userCount == 0) {
                addCompanyUser(userDetailJON);
            }
        } catch (Exception e) {
            LOGGER.info("获取信用户的详情异常:{}", e.getLocalizedMessage());
        }
    }

    public int getCompanyUserCount(String weChatUserId) {
        UserInfo userInfo = new UserInfo();
        userInfo.setWechatuserid(weChatUserId);
        userInfo.setIsDeleted(Constants.DEL_FLAG);
        int count = userInfoService.selectCount(QueryUtil.condition(userInfo));
        return count;
    }

    @Transactional(rollbackFor = Exception.class)
    public void addCompanyUser(JSONObject userDetailJON) {
        UserInfo userInfo = new UserInfo();
        userInfo.setId(idGenerator.generate());
        userInfo.setWechatuserid(userDetailJON.getString("userid"));
        userInfo.setNickname(userDetailJON.getString("name"));
        userInfo.setMobile(userDetailJON.getString("mobile"));
        userInfo.setEmail(userDetailJON.getString("email"));
        userInfo.setHeadimgurl(userDetailJON.getString("avatar"));
        userInfo.setQrCode(userDetailJON.getString("qr_code"));
        userInfo.setCreateTime(new Date());
        userInfo.setUserType(Constants.DEL_FLAG);

        boolean insertResult = userInfoService.insert(userInfo);
        LOGGER.info("企业微信用户新增结果：{}", insertResult);
    }

    /**
     * 小程序 新增用户
     *
     * @param openId
     */
    public void addMiniProgramUserInfo(String openId, String unionId) {
        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                userInfoService.saveMiniProgramUser(openId, unionId);
            }
        });
    }


    public void taskUpdateMiniUserInfo(Long accountId, String headPortrait) {
        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                updateMiniUserInfo(accountId, headPortrait);
            }
        });
    }

    private void updateMiniUserInfo(Long accountId, String headPortrait) {
        try {
            AccountVO accountVO = new AccountVO();
            accountVO.setId(accountId);
            accountVO.setHeadPortrait(headPortrait);
            Boolean updateAccount = accountClient.updateAccount(accountVO);
            LOGGER.info("异步小程序用户头像结果：{}", updateAccount ? "成功" : "失败");
        } catch (Exception e) {
            LOGGER.info("企业信息更新失败！{}", e.getMessage());
        }
    }

    private void updateAccountInfo(String wechatUserId, String nickName, String headImgUrl) {
        UserInfo userInfo = new UserInfo();
        userInfo.setWechatuserid(wechatUserId);
        userInfo.setHeadimgurl(headImgUrl);
        userInfo.setNickname(nickName);
        userInfoService.updateUserInfo(userInfo);

    }


    public void taskAddProviderUserInfo(UserInfo userInfo) {
        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                try {
                    int userCount = getCompanyUserCount(userInfo.getWechatuserid());
                    userInfo.setUserType(5);
                    LOGGER.info("返回的结果 userCount：{}", userCount);
                    if (userCount == 0) {
                        boolean insert = userInfoService.insert(userInfo);
                        LOGGER.info("返回新增结果：{}", insert);
                    }
                } catch (Exception e) {
                    LOGGER.info("获取信用户的详情异常:{}", e.getLocalizedMessage());
                }
            }
        });
    }
}
